# -*- coding: UTF-8 -*-
__author__ = 'Aki Mäkinen'

import os

from unittest import TestCase
from pettifogger.utils.yaml import yaml

from .jobs.needs import no_undefined_needs, no_circular_needs


def load_test_data(name):
    with open(os.path.join(os.path.dirname(os.path.realpath(__file__)), f"./testdata/jobs/{name}.yml")) as file:
        data = yaml.load(file)
    return data


class NeedsTests(TestCase):
    def test_missing_needs(self):
        test_data = load_test_data("missing_needs")
        for name, job in test_data["jobs"].items():
            ok, errors = no_undefined_needs(job, test_data["jobs"].keys())
            assert ok == False and len(errors) == 1

    def test_cyclic_needs(self):
        test_data = load_test_data("circular_needs")
        ok = no_circular_needs(test_data["jobs"])
        assert ok == False


# -*- coding: UTF-8 -*-
__author__ = 'Aki Mäkinen'

import re


class Env(dict):
    key_re = re.compile(r"^[a-zA-Z_][a-zA-Z0-9_]*$")
    ENV_VAR_REGEX = re.compile(r"\${{0,1}([a-zA-Z_][a-zA-Z0-9_]*)}{0,1}")
    SUSPECTED_ENV_VARS_REGEX = re.compile(r"(?<!\$)\b([A-Z_][A-Z0-9_]*|{[A-Z_][A-Z0-9_]*})\b")
    ENV_VAR_DEFINITION_REGEX = re.compile(r"{{0,1}(?P<key>[a-zA-Z_][a-zA-Z0-9_]*)}{0,1}=(?P<value>.*)")
    # SUSPECTED_ENV_VARS_REGEX = r"(?<!\$)\{{0,1}\b([A-Z_][A-Z0-9_]*)\}{0,1}\b"

    def __init__(self, env, *args, **kwargs):
        for key, value in env.items():
            if not Env.key_re.match(key):
                print(f"Key {key} does not follow the standard *nix environment variable format")
        super().__init__(**env, **kwargs)


# -*- coding: UTF-8 -*-
__author__ = 'Aki Mäkinen'

import shlex

from colorama import Fore

from pettifogger.workflow.structure.env import Env
from pettifogger.utils.configuration import get_global_configuration


class Step:
    def __init__(self, definition, context, index, parent_job_id, *args, **kwargs):
        self.raw_definition = definition
        self.parent_job_id = parent_job_id
        self.index = index
        self.log_source_id = f"Step {self.index}"
        self.id = definition.get("id")
        setattr(self, "if", definition.get("if"))
        self.name = definition.get("name")
        self.uses = definition.get("uses")
        self.run = definition.get("run")
        setattr(self, "when", definition.get("when"))
        self.continue_on_error = definition.get("continue-on-error")
        self.timeout_minutes = definition.get("timeout-minutes")

        self.env = Env({**context.get("env", {}), **definition.get("env", {})})

        self.configuration = get_global_configuration()

    def __handle_run_step_line_tokens(self, line, logger):
        shlex_instance = shlex.shlex(line, posix=True, punctuation_chars=True)
        script_level_env_var_pairs = []
        command_level_env_var_pairs = []
        has_errors = False
        try:
            token_list = list(shlex_instance)
        except Exception as e:
            has_errors = True
            logger.error(f"Exception: {str(e)} when handling line '{line}'", source_id=self.log_source_id)
            return {}, {}, has_errors

        i = 0
        while i < len(token_list):
            if token_list[i] == "export":
                m = Env.ENV_VAR_DEFINITION_REGEX.match(token_list[i + 1])
                if m is None:
                    has_errors = True
                    logger.error("Token 'export' was not followed by environment variable definition on line '{line}'",
                                 source_id=self.log_source_id)
                else:
                    script_level_env_var_pairs.append(m.groupdict())
                i += 2
            elif (m := Env.ENV_VAR_DEFINITION_REGEX.match(token_list[i])):
                command_level_env_var_pairs.append(m.groupdict())
            i += 1
        return ({item["key"]: item["value"] for item in script_level_env_var_pairs},
                {item["key"]: item["value"] for item in command_level_env_var_pairs},
                has_errors)

    def __check_run_step_env_vars(self, logger):
        has_errors = False
        has_suspicions = False
        if self.run is not None:
            run_lines = self.run.splitlines()
            script_level_env_vars = {**self.env}
            line_buffer = []
            for raw_line in run_lines:
                stripped_line = raw_line.strip()
                if stripped_line.endswith("\\"):
                    line_buffer.append(stripped_line)
                    continue

                if len(line_buffer) > 0:
                    line_buffer.append(stripped_line)
                    line = "\n".join(line_buffer)
                    line_buffer = []
                else:
                    line = stripped_line

                new_script_level_env_vars, new_command_level_env_vars, had_errors = \
                    self.__handle_run_step_line_tokens(line, logger)
                if had_errors:
                    has_errors = True

                script_level_env_vars = {**script_level_env_vars, **new_script_level_env_vars}
                command_env_vars = {**script_level_env_vars, **new_command_level_env_vars}

                string_final_env_vars_list = ", ".join(command_env_vars.keys())
                detected_env_vars = Env.ENV_VAR_REGEX.findall(line)
                suspected_env_vars = Env.SUSPECTED_ENV_VARS_REGEX.findall(line)

                for item in detected_env_vars:
                    if item not in command_env_vars:
                        has_errors = True
                        logger.error(
                            f"Could not find variable '{item}' from environment variables ({string_final_env_vars_list}) on line '{line}'",
                            source_id=self.log_source_id
                        )
                    elif str(command_env_vars[item]).strip() == "":
                        has_errors = True
                        logger.error(f"Value of '{item}' is empty on line '{line}'", source_id=self.log_source_id)

                if self.configuration.fail_on_suspicions:
                    suspicion_logger = logger.error
                else:
                    suspicion_logger = logger.warn

                for item in suspected_env_vars:
                    if item not in command_env_vars:
                        has_suspicions = True
                        suspicion_logger(
                            f"Could not find variable '{item}' from environment variables ({string_final_env_vars_list}) on line '{line}'",
                            source_id=self.log_source_id
                        )
                    elif str(command_env_vars[item]).strip() == "":
                        has_suspicions = True
                        suspicion_logger(f"Value of '{item}' is empty on line '{line}'", source_id=self.log_source_id)

        return not has_errors or (self.configuration.fail_on_suspicions and has_suspicions)

    def __run_validator_and_log(self, validator, section_name, logger):
        logger.info(f"\u2600 {section_name}:")
        result = validator(logger.new())
        if result:
            logger.info(f"  Result: {Fore.LIGHTGREEN_EX} \u2714")
        else:
            logger.info(f"  Result: {Fore.LIGHTRED_EX} \u2718")
        print()
        return result

    def validate(self, logger):
        logger.info(f"\u2600 Validating step {self.index}:")
        section_logger = logger.new()
        env_var_result = self.__run_validator_and_log(self.__check_run_step_env_vars,
                                                      "Check for undefined environment variables", section_logger)
        result = [
            env_var_result
        ]
        return all(result)


# -*- coding: UTF-8 -*-
__author__ = 'Aki Mäkinen'

import json
import os

import networkx as nx
from jsonschema import Draft7Validator
from colorama import Fore, Style

from .defaults import Defaults
from .env import Env
from .jobs.job import Job
from .on import On


class Workflow:
    def __init__(self, definition):
        self.raw_definition = definition

        self.name = definition.get("name", "Unnamed workflow")
        self.permissions = definition.get("permissions")
        self.concurrency = definition.get("concurrency")

        self.on = On(definition.get("on"))
        self.env = Env(definition.get("env")) if "env" in definition else None
        self.defaults = Defaults(definition.get("defaults")) if "defaults" in definition else None

        context = {
            "env": definition.get("env", {}),
            "defaults": definition.get("defaults", {}),
            "concurrency": definition.get("concurrency", {}),
            "permissions": definition.get("permissions", {})
        }

        self.jobs = [Job(id, definition, context) for id, definition in definition.get("jobs", {}).items()]

    def __forms_dag(self, logger):
        nodes = [ job.id for job in self.jobs ]
        edges = [(origin, target.id) for target in self.jobs if target.needs is not None for origin in target.needs ]

        G = nx.MultiDiGraph()
        G.add_nodes_from(nodes)
        G.add_edges_from(edges)

        is_dag = nx.algorithms.dag.is_directed_acyclic_graph(G)

        if not is_dag:
            logger.error("The workflow jobs do not form a directed acyclic graph", source_id=self.name)

        return is_dag

    def __jobs_have_no_undefined_needs(self, logger):
        job_names = [job.id for job in self.jobs]
        jobs_with_needs = [job for job in self.jobs if job.needs is not None]

        for job in jobs_with_needs:
            for need in job.needs:
                if need not in job_names:
                    logger.error(f"Undefined need ({need}) in job {job.id}", source_id=job.id)


    def __validate_schema(self, logger):
        with open(os.path.join(os.path.dirname(os.path.realpath(__file__)), "..", "..", "utils", "schema.json")) as schema_file:
            __schema = json.load(schema_file)

        validator = Draft7Validator(__schema)
        valid = validator.is_valid(dict(self.raw_definition))
        for error in validator.iter_errors(dict(self.raw_definition)):
            logger.error(f"{error.message} (path: {'.'.join(error.path)})", source_id=f"{self.name} - schema")
        return valid

    def __run_validator_and_log(self, validator, section_name, logger):
        logger.info(f"\u2600 {section_name}:")
        result = validator(logger.new())
        if result:
            logger.info(f"  Result: {Fore.LIGHTGREEN_EX} \u2714")
        else:
            logger.info(f"  Result: {Fore.LIGHTRED_EX} \u2718")
        print()
        return result

    def validate(self, logger):
        logger.info(f"\u2600 Workflow: {self.name}")
        section_logger = logger.new()
        schema_result = self.__run_validator_and_log(self.__validate_schema, "Schema validation", section_logger)

        dag_result = self.__run_validator_and_log(self.__forms_dag, "Check that jobs for a directed acyclic graph",
                                                  section_logger)

        no_undefined_needs_result = self.__run_validator_and_log(self.__jobs_have_no_undefined_needs,
                                                                 "Check for undefined needs", section_logger)

        results = [
            schema_result,
            dag_result,
            no_undefined_needs_result
        ]

        section_logger.info("\u2600 Validating jobs:")
        for job in self.jobs:
            results.append(job.validate(section_logger.new()))
            print()
        return all(results)


# -*- coding: UTF-8 -*-
__author__ = 'Aki Mäkinen'

import os

from unittest import TestCase
from pettifogger.utils.yaml import yaml

from .root.schema import validate_against_schema


def load_test_data(name):
    with open(os.path.join(os.path.dirname(os.path.realpath(__file__)), f"./testdata/schema/{name}.yml")) as file:
        data = yaml.load(file)
    return data


class RootSchemaTests(TestCase):
    def test_missing_jobs(self):
        test_data = load_test_data("missing_jobs")
        ok, errors = validate_against_schema(test_data)
        assert ok == False and len(errors) == 1

    def test_missing_triggers(self):
        test_data = load_test_data("missing_triggers")
        ok, errors = validate_against_schema(test_data)
        assert ok == False and len(errors) == 1


# -*- coding: UTF-8 -*-
__author__ = 'Aki Mäkinen'
import shutil
import textwrap

from colorama import Fore, Style

class State:
    def __init__(self):
        self.errors = {}
        self.suspicions = {}


class StatefulLogger:

    def __init__(self, indent=0, level=0, state=None):
        self.indent = indent
        self.level = level
        self.state = State() if state is None else state

    def __log(self, msg, color, *, level=None):
        if level is None:
            level = self.level
        terminal_size = shutil.get_terminal_size((120, 20))
        width = max(int(terminal_size.columns / 2), 120)  # Because.... Hervantavakiot?
        indent = " " * self.indent * level
        sub_indent = " " * self.indent * (level + 1)
        wrapper = textwrap.TextWrapper(width=width,
                                       initial_indent=indent,
                                       subsequent_indent=sub_indent)

        msg = wrapper.fill(color + msg + Style.RESET_ALL)
        print(msg)

    def info(self, *args):
        msg = ", ".join(args)
        self.__log(msg, Fore.LIGHTCYAN_EX)

    def warn(self, *args, source_id=None):
        if source_id is None:
            print("Logger id cannot be None")
            exit(1)

        msg = ", ".join(args)

        if source_id not in self.state.suspicions:
            self.state.suspicions[source_id] = []
        self.state.suspicions[source_id].append(msg)

        self.__log(msg, Fore.LIGHTYELLOW_EX)

    def error(self, *args, source_id=None):
        if source_id is None:
            print("Logger id cannot be None")
            exit(1)

        msg = ", ".join(args)

        if source_id not in self.state.errors:
            self.state.errors[source_id] = []
        self.state.errors[source_id].append(msg)

        self.__log(msg, Fore.LIGHTRED_EX)

    def summary(self, **kwargs):
        print(f"""
************************************************************************************************************************
SUMMARY ({kwargs.get('filename', '')})        
************************************************************************************************************************
""")

        print(f"SUSPICIONS ({sum([len(v) for v in self.state.suspicions.values()])}):\n")
        for logger_id, msgs in self.state.suspicions.items():
            self.__log(logger_id + ":", Fore.LIGHTCYAN_EX, level=0)
            for msg in msgs:
                self.__log("\u2600 " + msg, Fore.LIGHTYELLOW_EX, level=1)

        print("------------------------\n")

        print(f"ERRORS ({ sum([len(v) for v in self.state.errors.values()]) }):\n")
        for logger_id, msgs in self.state.errors.items():
            self.__log(logger_id + ":", Fore.LIGHTCYAN_EX, level=0)
            for msg in msgs:
                self.__log("\u2600 " + msg, Fore.LIGHTRED_EX, level=1)

    def reset_state(self):
        self.state = State()
        self.state.errors = {}
        self.state.suspicions = {}

    def new(self):
        return StatefulLogger(
            indent=self.indent,
            level=self.level + 1,
            state=self.state
        )



# -*- coding: UTF-8 -*-
__author__ = 'Aki Mäkinen'

from colorama import Fore

from pettifogger.utils.configuration import get_global_configuration
from pettifogger.workflow.structure.jobs.container import Container
from pettifogger.workflow.structure.jobs.step import Step
from pettifogger.workflow.structure.jobs.strategy import Strategy
from pettifogger.workflow.structure.env import Env
from pettifogger.workflow.structure.defaults import Defaults


class Job:
    def __init__(self, id, definition, context):
        self.id = id
        self.name = definition.get("name", id)
        self.needs = definition.get("needs")
        self.runs_on = definition.get("runs-on")
        self.permissions = definition.get("permissions")
        self.concurrency = definition.get("concurrency")
        self.outputs = definition.get("outputs")
        setattr(self, "if", definition.get("if"))
        self.timeout_minutes = definition.get("timeout-minutes")

        self.strategy = Strategy(definition.get("strategy")) if "strategy" in definition else None
        self.container = Container(definition.get("container")) if "container" in definition else None
        self.services = Container(definition.get("services")) if "services" in definition else None

        self.env = Env({**context.get("env", {}), **definition.get("env", {})})
        self.defaults = Defaults({**context.get("defaults", {}), **definition.get("defaults", {})})

        self.local_context = {
            "env": {**context.get("env", {}), **definition.get("env", {})},
            "defaults": {**context.get("defaults", {}), **definition.get("defaults", {})},
            "concurrency": definition.get("concurrency", {}),
            "permissions": definition.get("permissions", {})
        }
        self.parent_context = context

        self.context = {
            "env": {**self.parent_context.get("env", {}), **self.local_context.get("env", {})},
            "defaults": {**self.parent_context.get("defaults", {}), **self.local_context.get("defaults", {})},
            "concurrency": {**self.parent_context.get("concurrency", {}), **self.local_context.get("concurrency", {})},
            "permissions": {**self.parent_context.get("permissions", {}), **self.local_context.get("permissions", {})}
        }

        self.steps = [Step(step, self.context, index, self.id) for index, step in enumerate(definition.get("steps", []))]

        self.configuration = get_global_configuration()

    def _has_checkout(self, logger):
        for step in self.steps:
            if step.uses is not None and "actions/checkout@" in step.uses:
                return True
        if self.configuration.fail_on_suspicions:
            suspicion_logger = logger.error
        else:
            suspicion_logger = logger.warn
        suspicion_logger("No checkout", source_id=self.id)

        return False

    def __run_validator_and_log(self, validator, section_name, logger):
        logger.info(f"\u2600 {section_name}:")
        result = validator(logger.new())
        if result:
            logger.info(f"  Result: {Fore.LIGHTGREEN_EX} \u2714")
        else:
            logger.info(f"  Result: {Fore.LIGHTRED_EX} \u2718")
        print()
        return result

    def validate(self, logger):
        logger.info(f"\u2600 Validating job {self.id}:")
        section_logger = logger.new()

        checkout_result = self.__run_validator_and_log(self._has_checkout, "Checking if job has checkout",
                                                       section_logger)

        results = [
            True if not self.configuration.fail_on_suspicions else checkout_result
        ]
        for step in self.steps:
            results.append(step.validate(logger.new()))

        return all(results)


# -*- coding: UTF-8 -*-
__author__ = 'Aki Mäkinen'

from ruamel.yaml import YAML
from ruamel.yaml.compat import StringIO

yaml=YAML(typ="rt") # Just to be explicit

def dict_to_yaml_str(source):
    stream = StringIO()
    yaml.dump(source, stream)
    return stream.getvalue()


class GlobalConfiguration:
    def __init__(self):
        self.suppress_suspicions = False
        self.no_fail_on_error = False
        self.fail_on_suspicions = False

__global_configuration = GlobalConfiguration()

def get_global_configuration():
    global __global_configuration
    return __global_configuration

def set_global_configuration(conf):
    global __global_configuration
    __global_configuration = conf


#!/usr/bin/env python3
import os
import sys
import time
import threading
from argparse import ArgumentParser

from watchdog.observers import Observer
from watchdog.events import FileSystemEventHandler
from pettifogger.utils.configuration import GlobalConfiguration, set_global_configuration
from pettifogger.utils.stateful_logger import StatefulLogger
from pettifogger.workflow.structure.root import Workflow
from pettifogger.utils.yaml import yaml

with open(os.path.join(os.path.dirname(os.path.realpath(__file__)), "../VERSION")) as version_file:
    version = version_file.read()

__version__ = version


class WorkflowWatcher(FileSystemEventHandler):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.watched = []
        self._lock = threading.Lock()

    def __release_lock_after_grace_period(self, grace):
        time.sleep(grace)
        self._lock.release()

    def add_to_watched(self, path):
        self.watched.append(path)

    def on_modified(self, event):
        super().on_modified(event)
        if event.src_path.endswith("~") or event.src_path not in self.watched or self._lock.locked():
            return
        self._lock.acquire()
        release = threading.Thread(target=self.__release_lock_after_grace_period, args=(2,))
        release.start()

        if event.is_directory:
            files = os.listdir(event.src_path)
        else:
            files = [event.src_path]
        files = list(map(lambda x: os.path.join(event.src_path, x), files))
        filtered_files = list(filter(lambda x: os.path.exists(x) and not x.endswith("~"), files))
        results = validate_files(filtered_files)

        if not all(results):
            print(f"\n\nWorkflow validation failed, please check the logs", file=sys.stderr)

        print("\n-----------------------------------------------------------------------------------\n")

def validate_files(files):
    results = []
    loggers = {}

    for file in files:
        with open(file) as workflow_file:
            data = yaml.load(workflow_file)
        wf = Workflow(data)

        logger = StatefulLogger(indent=4)
        loggers[file] = logger

        result = wf.validate(logger)
        results.append(result)

    for f, l in loggers.items():
        l.summary(filename=f)
        l.reset_state()

    return results

def main():
    parser = ArgumentParser()
    parser.add_argument("--version", action="store_true", help="Print pettifogger's version")
    parser.add_argument("--workflow", help="Path to a workflow file to validate")
    parser.add_argument("--suppress-suspicions", action="store_true", help="Suppresses the suspicion output")
    parser.add_argument("--fail-on-suspicions", action="store_true", help="Suspicions fail the validation")
    parser.add_argument("--no-fail-on-error", action="store_true", help="Errors will not fail the validation")
    parser.add_argument("--watch", action="store_true", help="Watch the file or workflow subdirectories detected automatically")
    args = parser.parse_args()

    if args.version:
        print(f"pettifogger {__version__}")
        exit(0)

    configuration = GlobalConfiguration()
    configuration.suppress_suspicions = args.suppress_suspicions
    configuration.fail_on_suspicions = args.fail_on_suspicions
    configuration.no_fail_on_error = args.no_fail_on_error
    set_global_configuration(configuration)

    watch_paths = set()

    if args.workflow is not None:
        files = [args.workflow]
        if args.watch:
            watch_paths.add(args.workflow)
    else:
        paths_with_files = []
        for path, _, files in os.walk(os.getcwd()):
            paths_with_files.append((path, files))
        filtered = list(filter(lambda x: os.path.join(".github", "workflows") in x[0], paths_with_files))
        if args.watch:
            watch_paths = [item[0] for item in filtered]
        files = [os.path.join(item[0], file) for item in filtered for file in item[1]]

    results = validate_files(files)

    if not args.watch:
        if not all(results):
            print(f"\n\nWorkflow validation failed, please check the logs", file=sys.stderr)
            if args.no_fail_on_error:
                exit(0)
            exit(1)
    else:
        event_handler = WorkflowWatcher()
        observer = Observer()

        for p in watch_paths:
            event_handler.add_to_watched(p)
            observer.schedule(event_handler, p)
        observer.start()

        try:
            while True:
                time.sleep(2)
        except KeyboardInterrupt:
            observer.unschedule_all()
            observer.stop()
        observer.join()

if __name__ == '__main__':
    main()



# -*- coding: UTF-8 -*-
__author__ = 'Aki Mäkinen'

import os

from unittest import TestCase
from pettifogger.utils.yaml import yaml

from .jobs.checkout import has_checkout


def load_test_data(name):
    with open(os.path.join(os.path.dirname(os.path.realpath(__file__)), f"./testdata/jobs/{name}.yml")) as file:
        data = yaml.load(file)
    return data


class CheckoutTests(TestCase):
    def test_checkout_negative(self):
        test_data = load_test_data("no_checkout")
        for name, job in test_data["jobs"].items():
            assert has_checkout(job) == False

    def test_checkout_positive(self):
        test_data = load_test_data("has_checkout")
        for name, job in test_data["jobs"].items():
            assert has_checkout(job) == True





# -*- coding: UTF-8 -*-
__author__ = 'Aki Mäkinen'

import os

from unittest import TestCase
from pettifogger.utils.yaml import yaml

from .jobs.env_variables import existence


def load_test_data(name):
    with open(os.path.join(os.path.dirname(os.path.realpath(__file__)), f"./testdata/jobs/{name}.yml")) as file:
        data = yaml.load(file)
    return data


class EnvVarTests(TestCase):
    def test_export(self):
        test_data = load_test_data("export_env_var")
        for _, job in test_data["jobs"].items():
            e, _ = existence(job, {})
            assert len(e.keys()) == 0

    def test_command_env_var(self):
        test_data = load_test_data("command_env_var")
        for _, job in test_data["jobs"].items():
            e, _ = existence(job, {})
            assert len(e.keys()) == 0

    def test_missing_env_var(self):
        test_data = load_test_data("missing_env_vars")
        for _, job in test_data["jobs"].items():
            e, _ = existence(job, {})
            keys = list(e.keys())
            assert len(keys) == 1

            assert len(e[keys[0]].messages) == 2


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pettifogger',
 'pettifogger.checks',
 'pettifogger.utils',
 'pettifogger.workflow',
 'pettifogger.workflow.structure',
 'pettifogger.workflow.structure.jobs']

package_data = \
{'': ['*'],
 'pettifogger': ['testdata/.github/workflows/*'],
 'pettifogger.checks': ['testdata/jobs/*', 'testdata/schema/*']}

install_requires = \
['colorama>=0.4.4,<0.5.0',
 'jsonschema>=3.2.0,<4.0.0',
 'networkx>=2.5.1,<3.0.0',
 'ruamel.yaml>=0.17.4,<0.18.0',
 'watchdog==2.1.0']

entry_points = \
{'console_scripts': ['pettifogger = pettifogger.main:main']}

setup_kwargs = {
    'name': 'pettifogger',
    'version': '0.3.1',
    'description': 'Github actions workflow validator',
    'long_description': None,
    'author': 'Aki Mäkinen',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://gitlab.com/blissfulreboot/python/pettifogger',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



# -*- coding: UTF-8 -*-
__author__ = 'Aki Mäkinen'

class Strategy:
    def __init__(self, definition):
        self.matrix = definition.get("matrix")
        self.fail_fast = definition.get("fail-fast")
        self.max_parallel = definition.get("max-parallel")


# -*- coding: UTF-8 -*-
__author__ = 'Aki Mäkinen'

from pettifogger.workflow.structure.env import Env


class Container:
    def __init__(self, definition, *args, **kwargs):
        self.image = definition.get("image")
        self.credentials = definition.get("credentials")
        self.ports = definition.get("ports")
        self.volumes = definition.get("volumes")
        self.options = definition.get("options")

        self.env = Env(definition.get("env")) if "env" in definition else None
